package org.antlr.tool;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;
import java.util.ArrayList;
import java.util.List;

public class GrammarSpelunker {
  protected String grammarFileName;
  
  protected String token;
  
  protected Scanner scanner;
  
  protected String grammarModifier;
  
  protected String grammarName;
  
  protected String tokenVocab;
  
  protected String language = "Java";
  
  protected String inputDirectory;
  
  protected List<String> importedGrammars;
  
  public GrammarSpelunker(String inputDirectory, String grammarFileName) {
    this.inputDirectory = inputDirectory;
    this.grammarFileName = grammarFileName;
  }
  
  void consume() throws IOException {
    this.token = this.scanner.nextToken();
  }
  
  protected void match(String expecting) throws IOException {
    if (this.token.equals(expecting)) {
      consume();
    } else {
      throw new Error("Error parsing " + this.grammarFileName + ": '" + this.token + "' not expected '" + expecting + "'");
    } 
  }
  
  public void parse() throws IOException {
    Reader r = new FileReader(((this.inputDirectory != null) ? (this.inputDirectory + File.separator) : "") + this.grammarFileName);
    BufferedReader br = new BufferedReader(r);
    try {
      this.scanner = new Scanner(br);
      consume();
      grammarHeader();
      while (this.token != null && !this.token.equals("@") && !this.token.equals(":") && !this.token.equals("import") && !this.token.equals("options"))
        consume(); 
      if (this.token.equals("options"))
        options(); 
      while (this.token != null && !this.token.equals("@") && !this.token.equals(":") && !this.token.equals("import"))
        consume(); 
      if (this.token.equals("import"))
        imports(); 
    } finally {
      if (br != null)
        br.close(); 
    } 
  }
  
  protected void grammarHeader() throws IOException {
    if (this.token == null)
      return; 
    if (this.token.equals("tree") || this.token.equals("parser") || this.token.equals("lexer")) {
      this.grammarModifier = this.token;
      consume();
    } 
    match("grammar");
    this.grammarName = this.token;
    consume();
  }
  
  protected void options() throws IOException {
    match("options");
    match("{");
    while (this.token != null && !this.token.equals("}")) {
      String name = this.token;
      consume();
      String value = this.token;
      consume();
      match(";");
      if (name.equals("tokenVocab"))
        this.tokenVocab = value; 
      if (name.equals("language"))
        this.language = value; 
    } 
    match("}");
  }
  
  protected void imports() throws IOException {
    match("import");
    this.importedGrammars = new ArrayList<String>();
    while (this.token != null && !this.token.equals(";")) {
      this.importedGrammars.add(this.token);
      consume();
    } 
    match(";");
    if (this.importedGrammars.isEmpty())
      this.importedGrammars = null; 
  }
  
  public String getGrammarModifier() {
    return this.grammarModifier;
  }
  
  public String getGrammarName() {
    return this.grammarName;
  }
  
  public String getTokenVocab() {
    return this.tokenVocab;
  }
  
  public String getLanguage() {
    return this.language;
  }
  
  public List<String> getImportedGrammars() {
    return this.importedGrammars;
  }
  
  public static void main(String[] args) throws IOException {
    org.antlr.tool.GrammarSpelunker g = new org.antlr.tool.GrammarSpelunker(".", args[0]);
    g.parse();
    System.out.println(g.grammarModifier + " grammar " + g.grammarName);
    System.out.println("language=" + g.language);
    System.out.println("tokenVocab=" + g.tokenVocab);
    System.out.println("imports=" + g.importedGrammars);
  }
}
